UNEXPECTED_FIELD = "Unexpected field"
UNEXPECTED_FIELDS = "Unexpected fields: {unexpected_fields}"
MISSING_REQUIRED_FIELD = "Missing required field"
VALUE_NOT_IN_CHOICES = "Value not in choices"
FIELD_TOO_LONG = "Field too long"
FIELD_TOO_SHORT = "Field too short"
INVALID_STRING = "Invalid string"
REGEX_NOT_MATCHED = "Regex not matched"
INVALID_DATETIME = "Invalid datetime"
INVALID_DATE = "Invalid date"
INVALID_INT = "Invalid int"
INVALID_FLOAT = "Invalid float"
INVALID_LONG = "Invalid long"
INVALID_DECIMAL = "Invalid decimal"
INVALID_BOOLEAN = "Invalid boolean"
NUMBER_TOO_HIGH = "Number too high"
NUMBER_TOO_LOW = "Number too low"
INVALID_EMAIL = "Invalid email"
INVALID_URL = "Invalid URL"
INVALID_UUID = "Invalid UUID"
INVALID_IPV4 = "Invalid IPv4"
INVALID_IPV6 = "Invalid IPv6"
INVALID_HOSTNAME = "Invalid hostname"
INVALID_URI = "Invalid URI"
INVALID_REGEX = "Invalid regex"
INVALID_COLOR = "Invalid color"
INVALID_HASH_LENGTH = "Invalid hash length"
INVALID_HASH = "Invalid hash"
LIST_TOO_SHORT = "List too short"
LIST_TOO_LONG = "List too long"
INVALID_LIST = "Invalid list"
INVALID_DICT = "Invalid dict"
INVALID_CHILD_CONTENT = "Invalid child content"
INVALID_NUMBER = "Invalid number"
INVALID_OBJECT = "Invalid object"
INVALID_ARRAY = "Invalid array"
NOT_NULL = "Not null"
NOT_VALID_UNDER_ANY_SCHEMA = "Not valid under any schema"
VALID_FOR_SEVERAL_EXCLUSIVE_SCHEMAS = "Valid for several exclusive schemas"
TOO_MANY_ITEMS = "Too many items"
MISSING_DEPENDENT_FIELD = "Missing dependent field"
TOO_MANY_PROPERTIES = "Too many properties"
NOT_ENOUGH_PROPERTIES = "Not enough properties"
NOT_MULTIPLE_OF = "Not multiple of"
NOT_ALLOWED_VALUE = "Not allowed value"
NOT_UNIQUE = "Not unique"
