{% import 'slack/spider/notifier/macros.jinja' as renderer with context %}
:skull: `{{ renderer.render_spider_name() }}` *Detected errors on running spider!*{{ renderer.render_job_url() }} _(errors={{ result.monitors_failed_results|length }})_
