/*--------------------------
 Reset
--------------------------*/
html{font-size:100%}
button,input,select,textarea{margin:0;font-size:100%;vertical-align:middle}
button,input{line-height:normal}
textarea{overflow:auto;vertical-align:top}
body{margin:0;font-family:"Segoe UI","Helvetica Neue",Helvetica,Arial,sans-serif;font-size:13px;line-height:20px;color:#333;background-color:#fff}
a{color:#0063ca;text-decoration:none}
a:hover{color:#003e7e;text-decoration:underline}
p{margin:0 0 10px}
small{font-size:85%}
strong{font-weight:bold}
em{font-style:italic}
cite{font-style:normal}
h1,h2,h3,h4,h5,h6{margin:10px 0;font-family:inherit;font-weight:bold;line-height:1;color:inherit;}
h1 small,h2 small,h3 small,h4 small,h5 small,h6 small{font-weight:normal;line-height:1;color:#999}
h1{font-size:36px;line-height:40px}
h2{font-size:30px;line-height:40px}
h3{font-size:24px;line-height:40px}
h4{font-size:18px;line-height:20px}
h5{font-size:14px;line-height:20px}
h6{font-size:12px;line-height:20px}
h1 small{font-size:24px}
h2 small{font-size:18px}
h3 small{font-size:14px}
h4 small{font-size:14px}
ul,ol{padding:0;margin:0 0 10px 25px}
ul ul,ul ol,ol ol,ol ul{margin-bottom:0}
li{line-height:20px}
hr{margin:20px 0;border:0;border-top:1px solid #eee;border-bottom:1px solid #fff}
td{margin:0;padding:0;}
code,pre{padding:0 3px 2px;font-family:Monaco,Menlo,Consolas,"Courier New",monospace;font-size:11px;color:#333;border-radius:3px}
code{padding:2px 4px;color:#d14;background-color:#f7f7f9;border:1px solid #e1e1e8}
pre{display:block;padding:9.5px;margin:0 0 10px;font-size:12px;line-height:20px;white-space:pre;white-space:pre-wrap;background-color:#f5f5f5;border:1px solid #ccc;border:1px solid rgba(0,0,0,0.15);border-radius:4px}
pre code{padding:0;color:inherit;background-color:transparent;border:0}
table{max-width:100%;background-color:transparent;border-collapse:collapse;border-spacing:0}

/*--------------------------
 Buttons
--------------------------*/
.btn{background:#999;border-radius:6px;border:0;color:#fff;cursor:pointer;display:inline-block;margin:2px 0;padding:12px 30px 14px;}
.btn{font-size: 16px;}
.btn:hover{background:#888;text-decoration: none;color:#fff;}
.btn:active,
.btn:focus{background:#777;text-decoration: none;color:#fff;}
.btn-blue{background-color:#09d}
.btn-blue:hover{background-color:#0ae}
.btn-blue:active,
.btn-blue:focus{background-color:#09d}
.btn-green{background-color:#2b4}
.btn-green:hover{background-color:#3c5}
.btn-green:active,
.btn-green:focus{background-color:#3c5}
.btn-red{background-color:#c22}
.btn-red:hover{background-color:#d33}
.btn-red:active,
.btn-red:focus{background-color:#d33}
.btn-sm{border-radius:4px;padding:8px 10px;font-size: 14px;}

/*--------------------------
 Alerts
--------------------------*/
.alert {padding: 15px;margin-bottom: 20px;border: 1px solid transparent;border-radius: 4px;}
.alert-warning {color: #8a6d3b;background-color: #fcf8e3;border-color: #faebcc;}

/*--------------------------
 Labels
--------------------------*/
.label {border-radius: 3px;font-size: 12px;font-weight: bold;color: #ffffff;padding: 2px 6px 2px 6px;line-height: 20px;height: 20px;}
.label-gray {background-color: #999;}
.label-blue {background-color: #3a87ad;}

/*--------------------------
 Badge
--------------------------*/
.badge {padding: 2px 9px 2px 9px;border-radius: 9px;font-size: 12px;font-weight: 700;line-height: 14px;color: #fff;vertical-align: baseline;white-space: nowrap;background-color: #999;text-align: center;}
.badge-green {background-color: #468847}
.badge-red {background-color: #b94a48}
/*--------------------------
 Icons
--------------------------*/
.icon,.icon-big {display:inline-block;}
.icon {width:34px;height:34px;}
.icon-big {width:140px;height:140px;}


