{% import 'slack/spider/notifier/macros.jinja' as renderer with context %}
{% if monitors_failed %}
    :skull: `{{ renderer.render_spider_name() }}` *spider finished with errors!*{{ renderer.render_url() }} _(errors={{ result.monitors_failed_results|length }})_
{% else %}
    :thumbsup: `{{ renderer.render_spider_name() }}` *spider finished!*{{ renderer.render_url() }}
{% endif %}